// ******************************************************************************
// * Licensed Materials - Property of IBM
// * 5724-W86, 5724-X63
// *  Copyright IBM Corporation 2008, 2012. All Rights Reserved.
// * US Government Users Restricted Rights- Use, duplication or disclosure
// * restricted by GSA ADP Schedule Contract with IBM Corp.
// ******************************************************************************
// *
// ******************************************************************************

/**
 * Creates the input schema and returns itself. 
 * @return PV.Schema the schema to be processed named CSV
 */
function createAlcatel5529SdcInputSchema()
{
	var inputSchema=PV.TextInputSchema(
		"CSV",
		{
			filePeriod: 900,
			heartbeatOnLastTimestamp: false,
			multiFileJoin: true,
			// multiFileJoin: false,
                        noNewInputLimit: 2,

			parseURI: function(inputDescriptor) {
				inputDescriptor.timestamp = dateFromFilename(inputDescriptor.localName);
				inputDescriptor.elementName = elementNameFromFilename(inputDescriptor.localName);
				logMessage(5, logComponent, "parseURI",
					"Timestamp from " + inputDescriptor.localName + " is " + inputDescriptor.timestamp);
			} // End function parseURI
		},
		PV.DelimitedRecordSchema(
			"data",
			{
				fieldSeparator: ",",
				recordSeparator: "\r",
				maxOccurs: Infinity,
				hasHeaderRecord: false,
				onRead: processRecord
			},
			PV.StringFieldSchema("Timestamp"),
			PV.StringFieldSchema("ObjectID"),
			PV.StringFieldSchema("CounterID"),
			PV.StringFieldSchema("Value")
		) // End DelimitedRecordSchema
	); // End TextInputSchema
	return inputSchema;
}// End createRootOperator
		
/**
 * Function elementFromFilename <br>
 * Create an element from a filename.  Naming convention is:
 * 	${DSLAM}_${DATE}.csv
 * @param {String} Name of the file to extact from
 * @return {String} the name of the element
 */
function elementNameFromFilename(filename) {
	var name = filename.substring(0, filename.lastIndexOf("_"));
	var myelement = name;
	return name;
} // End function elementFromFilename

/**
 * Function isDailyPM <br>
 * returns true if this counter rolls over at the top of the day
 * @param {String} the id of the counter to verify
 * @return {boolean} 
 */
function isDailyPM(counterID) {
        for (var i=0; i<dailyPMs.length; i++)
        	{
                if (dailyPMs[i]==counterID)
                        return true;
		} // end for
	return false;
	} // end function isDailyPM

/**
 *  Function dateFromFilename <br>
 * Determine the date from the filename. Naming convention is: <br>
 * 	${DSLAM}_${DATE}.csv <br>
 * Where DATE is specified as: <br>
 * 	yyyy-MM-dd-HH
 * @param {String} filename - name of the file to get the date from
 * @return {Date} date - the date this file references
 */
function dateFromFilename(filename) {
        // ../../UBA.2.100/ALCATEL_5529_SDC/done/CSV/jumbo-file_1_2010-04-03-14-46.csv
	// var dateString = filename.slice(filename.lastIndexOf("_") + 1, filename.lastIndexOf("."));
	var dateString = filename.slice(filename.lastIndexOf("_") + 1, filename.lastIndexOf("."));
	var dateParts = dateString.split("-");

	if (dateParts.length == 4)
		{
		var date = new Date(dateParts[0], dateParts[1] - 1, dateParts[2], dateParts[3]);
		}

        if (dateParts.length == 5)
                {
                var date = new Date(dateParts[0], dateParts[1] - 1, dateParts[2], dateParts[3], dateParts[4]);

		// THIS CODE WAS REQUIRED PRIOR TO THE JAVA LOAD BALANCER
                // dateParts[4] contains the minutes, however, each 15 minute file is actually dated with
                // the 16th minute. So we need to subtract 1 minute from the file to get the actual 15
                // minute block that it has data for.

		// now, subtract 1 minute from it. 1 minute == 60 * 1000 ms, so we will convert the date to a number
		// then subtract 1000 from it, then convet back to a date again

		//var dateAsNumber = date.getTime();
		//dateAsNumber = dateAsNumber - (1000 * 60);
		//date.setTime(dateAsNumber);
                }
	
	currentTimeStamp = date;
	return date;
} // End function dateFromFilename

/**
 * Function processRecord <br>
 * This function is called for each record in the CSV file. 
 * It creates a sub-element within the global element.
 * Properties and Metrics are added using the APModeler library.
 * @param {Object} record
 * @param {Object} inputDescriptor
 */
function processRecord(record, inputDescriptor) {
	// 4.3V un explained comments, not touched in 4.3X
	// Ports have an ObjectID of the following form:
	// rack-shelf-slot-port
//	var regExp = new RegExp("^[0-9]+\-[0-9]+\-[0-9]+\-[0-9]+$");
//	if (regExp.exec(record.ObjectID) == null) 
//		return; 
	
	// There are three ways to do this (version dependent)
	// use PV.currentInputDescriptor (bad)
	// pass the descriptor to the function
	// use this.rootSchema.currentObject (newest)
	//      ^       ^           ^
	//      |       |           - The current object of the schema (Input Descriptor)
	//      |       -the rootSchema for current schema (TextInputSchema)
	//      - The current schema (RecordSchema)
	// Adding the following line to create the mynewlement variable in processRecord
	var mynewelement;
	var descriptor;
	if(isValueValid(this.rootSchema)) 
	{
		if(isValueValid(this.rootSchema.currentObject))
		{
			descriptor=this.rootSchema.currentObject;
		}
	}
	if(! isValueValid(descriptor) && isValueValid(inputDescriptor))
	{
		descriptor=inputDescriptor;
	}
	if(!isValueValid(descriptor)) 
	{
		descriptor=PV.currentInputDescriptor;
	}

        if ( isValid(record.Timestamp) && record.Timestamp.indexOf("Filename") != -1  ) {
           // we have an embedded filename in this line. 
           // we need to extract the element name from it.
           var fname = record.Timestamp.split(":").pop();
           ename = elementNameFromFilename(fname);
           descriptor.elementName = ename;
           descriptor.timestamp = dateFromFilename(fname);
             var myelement = ename;
           return;
        }
	
	record.element= new elementInRecord(descriptor.elementName);
	//record.element.timestamp = descriptor.timestamp;
	
	// Create the subelement		
	record.subelement = subelementFromRecord(record,descriptor.timestamp);
        if ( record.CounterID == "90011" ) {
            logMessage(4, "ETH SPEED", "Speed for subelement: " + record.subelement + " = " + record.Value);
            ethSpeedCache[record.subelement] = record.Value;
            logMessage(4, "ETH SPEED", "Stored speed in ethSpeedCache");
        }
	
	// add the timestamp to the record so addMetric can use it
	record.Timestamp = descriptor.timestamp;
	record.timestamp = descriptor.timestamp.asUTCSeconds();
        record.descriptorName = descriptor.localName;
	
	makeValueFloatIfPossible(record);

	//You must have an objectID or this record is useless
	if( isValid(record.ObjectID)) {
			this.emitTuple(record);	
	} else {
		logMessage(1, logComponent, "processRecord",
					"Record not used: " + record +
					", Timestamp: " + record.Timestamp +
					", ObjectID:" + record.ObjectID +
					", CounterID:" + record.CounterID +
					", Value:" + record.Value);
	}
} // End function processRecord

/**
 * Function subelementFromRecord<br>
 * Create a new subelement if different to the global subelement
 * @param {PV.Record} record
 * @param {Date} timestamp
 */
function subelementFromRecord(record,timestamp) {
	var subelement_object = "";

	// var regExp = new RegExp("^[0-9]+\-[0-9]+\-[0-9]+\-[0-9]+$");
        // now a global: var seRegExp = new RegExp("^[0-9]+\-[0-9]+\-[0-9]+\-[0-9]+$");

	//Change in 1.3D so that PON resource do not get discarded. (PON ONLY have 1 CounterID, 11070)
	//if (cardlevelreports == "TRUE" && seRegExp.exec(record.ObjectID) != null)
	if( cardlevelreports == "TRUE" && seRegExp.exec(record.ObjectID) != null && record.CounterID != 11070)
		{
		subelement_object = record.ObjectID.substring(0, record.ObjectID.lastIndexOf("-"));	
		}
	else
		subelement_object = record.ObjectID;

	var seName = record.element.name + ":" + subelement_object;
	
	var newSubelement = new subelementInRecord(seName);		
	newSubelement.instance = subelement_object;

	// Change for PMR 27081,012,649 and APAR IV19127, the CFHD related tablespaces are growing incrementally issue
	// newSubelement.timestamp = timestamp;
	
	logMessage(7, logComponent, "subelementFromRecord",
		"Created new subelement " + newSubelement.name);
	
	return newSubelement;
} // End function subelementFromRecord

/**
 * Function that enhances an element object to be placed on the record
 * @param {Object} elementName
 * @return {APObject} element Object
 */
function elementInRecord(elementName) {
	var result=ElementInRecord.clone().mixIn({
		name:		elementName
	});
	return result;
} // end class elementInRecord

/**
 * Define the APObject
 */
function APObject() {
}
/**
 * Associate the prototype of APObject to be the same as the object prototype
 * this may be interpreter specific (SMEE)
 */
APObject.prototype=Object.prototype;

/**
 * Take the hash provided and associate all the keys with children of the current
 * APObject
 * @param {Object} hash
 * @return {APObject} self
 */
APObject.prototype.mixIn = function (hash) {
	for (var key in hash) if (hash.hasOwnProperty(key)) {
		this[key]=hash[key];
	}
	return this;
}

/**
 * Add cloning to the existing object
 * @return {APObject} clone of original object
 */
APObject.prototype.clone = function() {
	var constructor = function() { this.constructor = arguments.callee; };
	constructor.prototype=this;
	return new constructor();
}

/**
 * Defines the ElementInRecord object as a "type" of APObject with some additional static variables
 * @classDescription Element in record is an object that contains the well known parameters for creating an APObject that can be passed without the overhead of a PV.Element it's designed to be accessed later when there is access to the operator
 */
ElementInRecord=APObject.clone().mixIn({
	state: true,
	origin: "Alcatel-Lucent 5529 SDC",
	toString: function (){ return "elementInRecord: " + this.name;}
});

/**
 * Defines the SubelementInRecord object as a "type" of APObject with some additional static variables
 * @classDescription Subelement in record is an object that contains the well known parameters for creating an APObject that can be passed without the overhead of a PV.Subelement.  it's designed to be accessed later when there is access to the operator
 */
SubelementInRecord=APObject.clone().mixIn({
	state: true,
	instance: 'None',
	family: "Alcatel5529SDC_Port",
	origin: "Alcatel-Lucent 5529 SDC",
	toString: function () { 
		return "subelementInRecord: " + this.name;
	}
});


/**
 * Function that enhances a subelement object to be placed on the record
 * @param {String} subelementName
 * @return {SubElementInRecord} subelement Object
 */
function subelementInRecord(subelementName){
	var result=SubelementInRecord.clone().mixIn({
		instance:	subelementName,
		invariant:	subelementName,
		label:		subelementName,
		name:		subelementName
	});
	return result;
}

/**
 * This takes the record.value and saves over it as a float object if possible
 * @param {PV.Record} record
 */
function makeValueFloatIfPossible(record){
	var valueObject;
	var logSciNotation = false;
	if(!isValid(record.Value)){
		return;
	}
	if( record.Value.indexOf("E") > 0 && record.Value.indexOf(".") > 0)
		{
		logSciNotation = true;
		valueObject = parseInt(record.Value);
		if(logSciNotation) logMessage(5, logComponent, "makeValueFloat..",	"Int parsed value is: " + valueObject);
		if( valueObject != NaN && valueObject != undefined ) {
			record.Value = valueObject;
			return;
		}
		valueObject = parseFloat(record.Value);
		if(logSciNotation) logMessage(5, logComponent, "makeValueFloat..",	"Float parsed value is: " + valueObject);
		if( valueObject != NaN && valueObject != undefined) {
			record.Value = valueObject;
			return;
		}
		if(logSciNotation) logMessage(5, logComponent, "makeValueFloat..",	"Nan for sci-notation value is: " + record.Value);
	}
	
}
